@@ -52,28 +52,3 @@ class Type(object):
         raised.
         """
         
-    def pserialize(self, node, value):
-        """ Partially serialize a value, ignoring any missing
-        components.
-
-        The description of the ``node`` and ``value`` arguments are
-        the same as those provided to ``serialize``.
-
-        The return value and behavior of any partial serialization is
-        completely type-dependent.  If partial serialization is not
-        applicable for a type, this method will usually be an alias
-        for that type's 'serialize' method.
-        """
-
-    def pdeserialize(self, node, value):
-        """ Partially deserialize a value, ignoring any missing
-        components.
-
-        The description of the ``node`` and ``value`` arguments are
-        the same as those provided to ``deserialize``.
-
-        The return value and behavior of any partial deserialization
-        is completely type-dependent.  If partial deserialization is
-        not applicable for a type, this method will usually be an
-        alias for that type's 'deserialize' method.
-        """
