@@ -189,7 +189,7 @@ class TestRange(unittest.TestCase):
     def test_min_failure(self):
         validator = self._makeOne(min=1)
         e = invalid_exc(validator, None, 0)
-        self.assertEqual(e.msg, '0 is less than minimum value 1')
+        self.assertEqual(e.msg.interpolate(), '0 is less than minimum value 1')
 
     def test_min_failure_msg_override(self):
         validator = self._makeOne(min=1, min_err='wrong')
@@ -199,7 +199,8 @@ class TestRange(unittest.TestCase):
     def test_max_failure(self):
         validator = self._makeOne(max=1)
         e = invalid_exc(validator, None, 2)
-        self.assertEqual(e.msg, '2 is greater than maximum value 1')
+        self.assertEqual(e.msg.interpolate(),
+                         '2 is greater than maximum value 1')
 
     def test_max_failure_msg_override(self):
         validator = self._makeOne(max=1, max_err='wrong')
@@ -272,12 +273,12 @@ class TestLength(unittest.TestCase):
     def test_min_failure(self):
         validator = self._makeOne(min=1)
         e = invalid_exc(validator, None, '')
-        self.assertEqual(e.msg, 'Shorter than minimum length 1')
+        self.assertEqual(e.msg.interpolate(), 'Shorter than minimum length 1')
 
     def test_max_failure(self):
         validator = self._makeOne(max=1)
         e = invalid_exc(validator, None, 'ab')
-        self.assertEqual(e.msg, 'Longer than maximum length 1')
+        self.assertEqual(e.msg.interpolate(), 'Longer than maximum length 1')
 
 class TestOneOf(unittest.TestCase):
     def _makeOne(self, values):
@@ -291,7 +292,8 @@ class TestOneOf(unittest.TestCase):
     def test_failure(self):
         validator = self._makeOne([1, 2])
         e = invalid_exc(validator, None, None)
-        self.assertEqual(e.msg, '"None" is not one of 1, 2')
+        self.assertEqual(e.msg.interpolate(), '"None" is not one of 1, 2')
+
 
 class TestMapping(unittest.TestCase):
     def _makeOne(self, *arg, **kw):
@@ -314,7 +316,7 @@ class TestMapping(unittest.TestCase):
         typ = self._makeOne()
         e = invalid_exc(typ.deserialize, node, None)
         self.failUnless(
-            e.msg.startswith('None is not a mapping type'))
+            e.msg.interpolate().startswith('"None" is not a mapping type'))
 
     def test_deserialize_no_subnodes(self):
         node = DummySchemaNode(None)
@@ -334,7 +336,9 @@ class TestMapping(unittest.TestCase):
         node.children = [DummySchemaNode(None, name='a')]
         typ = self._makeOne(unknown='raise')
         e = invalid_exc(typ.deserialize, node, {'a':1, 'b':2})
-        self.assertEqual(e.msg, "Unrecognized keys in mapping: {'b': 2}")
+        self.assertEqual(e.msg.interpolate(),
+                         "Unrecognized keys in mapping: \"{'b': 2}\"")
+
 
     def test_deserialize_unknown_preserve(self):
         node = DummySchemaNode(None)
@@ -372,7 +376,8 @@ class TestMapping(unittest.TestCase):
             ]
         typ = self._makeOne()
         e = invalid_exc(typ.deserialize, node, {'a':1})
-        self.assertEqual(e.children[0].msg, '"b" is required but missing')
+        self.assertEqual(e.children[0].msg.interpolate(),
+                         '"b" is required but missing')
 
     def test_deserialize_subnode_partial(self):
         node = DummySchemaNode(None)
@@ -389,7 +394,7 @@ class TestMapping(unittest.TestCase):
         typ = self._makeOne()
         e = invalid_exc(typ.serialize, node, None)
         self.failUnless(
-            e.msg.startswith('None is not a mapping type'))
+            e.msg.interpolate().startswith('"None" is not a mapping type'))
 
     def test_serialize_no_subnodes(self):
         node = DummySchemaNode(None)
@@ -443,8 +448,8 @@ class TestTuple(unittest.TestCase):
         typ = self._makeOne()
         e = invalid_exc(typ.deserialize, node, None)
         self.assertEqual(
-            e.msg,
-            'None is not iterable')
+            e.msg.interpolate(),
+            '"None" is not iterable')
         self.assertEqual(e.node, node)
 
     def test_deserialize_no_subnodes(self):
@@ -465,16 +470,16 @@ class TestTuple(unittest.TestCase):
         node.children = [DummySchemaNode(None, name='a')]
         typ = self._makeOne()
         e = invalid_exc(typ.deserialize, node, ('a','b'))
-        self.assertEqual(e.msg,
-           "('a', 'b') has an incorrect number of elements (expected 1, was 2)")
+        self.assertEqual(e.msg.interpolate(),
+      "\"('a', 'b')\" has an incorrect number of elements (expected 1, was 2)")
 
     def test_deserialize_toosmall(self):
         node = DummySchemaNode(None)
         node.children = [DummySchemaNode(None, name='a')]
         typ = self._makeOne()
         e = invalid_exc(typ.deserialize, node, ())
-        self.assertEqual(e.msg,
-           "() has an incorrect number of elements (expected 1, was 0)")
+        self.assertEqual(e.msg.interpolate(),
+           '"()" has an incorrect number of elements (expected 1, was 0)')
 
     def test_deserialize_subnodes_raise(self):
         node = DummySchemaNode(None)
@@ -492,8 +497,8 @@ class TestTuple(unittest.TestCase):
         typ = self._makeOne()
         e = invalid_exc(typ.serialize, node, None)
         self.assertEqual(
-            e.msg,
-            'None is not iterable')
+            e.msg.interpolate(),
+            '"None" is not iterable')
         self.assertEqual(e.node, node)
 
     def test_serialize_no_subnodes(self):
@@ -514,16 +519,17 @@ class TestTuple(unittest.TestCase):
         node.children = [DummySchemaNode(None, name='a')]
         typ = self._makeOne()
         e = invalid_exc(typ.serialize, node, ('a','b'))
-        self.assertEqual(e.msg,
-           "('a', 'b') has an incorrect number of elements (expected 1, was 2)")
+        self.assertEqual(e.msg.interpolate(),
+     "\"('a', 'b')\" has an incorrect number of elements (expected 1, was 2)")
 
     def test_serialize_toosmall(self):
         node = DummySchemaNode(None)
         node.children = [DummySchemaNode(None, name='a')]
         typ = self._makeOne()
         e = invalid_exc(typ.serialize, node, ())
-        self.assertEqual(e.msg,
-           "() has an incorrect number of elements (expected 1, was 0)")
+        self.assertEqual(e.msg.interpolate(),
+           '"()" has an incorrect number of elements (expected 1, was 0)'
+           )
 
     def test_serialize_subnodes_raise(self):
         node = DummySchemaNode(None)
@@ -552,8 +558,8 @@ class TestSequence(unittest.TestCase):
         node.children = [node]
         e = invalid_exc(typ.deserialize, node, None)
         self.assertEqual(
-            e.msg,
-            'None is not iterable')
+            e.msg.interpolate(),
+            '"None" is not iterable')
         self.assertEqual(e.node, node)
 
     def test_deserialize_not_iterable_accept_scalar(self):
@@ -592,8 +598,8 @@ class TestSequence(unittest.TestCase):
         node.children = [node]
         e = invalid_exc(typ.serialize, node, None)
         self.assertEqual(
-            e.msg,
-            'None is not iterable')
+            e.msg.interpolate(),
+            '"None" is not iterable')
         self.assertEqual(e.node, node)
 
     def test_serialize_not_iterable_accept_scalar(self):
@@ -626,9 +632,9 @@ class TestSequence(unittest.TestCase):
         self.assertEqual(len(e.children), 2)
 
 class TestString(unittest.TestCase):
-    def _makeOne(self, encoding='utf-8'):
+    def _makeOne(self, encoding='utf-8', allow_empty=False):
         from colander import String
-        return String(encoding)
+        return String(encoding, allow_empty)
 
     def test_alias(self):
         from colander import Str
@@ -649,6 +655,12 @@ class TestString(unittest.TestCase):
         result = typ.deserialize(node, val)
         self.assertEqual(result, 'default')
 
+    def test_deserialize_emptystring_allow_empty(self):
+        node = DummySchemaNode(None)
+        typ = self._makeOne(None, True)
+        result = typ.deserialize(node, '')
+        self.assertEqual(result, '')
+
     def test_deserialize_uncooperative(self):
         val = Uncooperative()
         node = DummySchemaNode(None)
@@ -894,14 +906,15 @@ class TestGlobalObject(unittest.TestCase):
         typ = self._makeOne()
         e = invalid_exc(typ._zope_dottedname_style, None, '.')
         self.assertEqual(
-            e.msg,
+            e.msg.interpolate(),
             'relative name "." irresolveable without package')
 
     def test_zope_dottedname_style_resolve_relative_nocurrentpackage(self):
         typ = self._makeOne()
         e = invalid_exc(typ._zope_dottedname_style, None, '.whatever')
         self.assertEqual(
-            e.msg, 'relative name ".whatever" irresolveable without package')
+            e.msg.interpolate(),
+            'relative name ".whatever" irresolveable without package')
 
     def test_zope_dottedname_style_irrresolveable_relative(self):
         import colander.tests
@@ -971,7 +984,7 @@ class TestGlobalObject(unittest.TestCase):
     def test_deserialize_not_a_string(self):
         typ = self._makeOne()
         e = invalid_exc(typ.deserialize, None, None)
-        self.assertEqual(e.msg, '"None" is not a string')
+        self.assertEqual(e.msg.interpolate(), '"None" is not a string')
 
     def test_deserialize_using_pkgresources_style(self):
         typ = self._makeOne()
@@ -986,7 +999,7 @@ class TestGlobalObject(unittest.TestCase):
     def test_deserialize_style_raises(self):
         typ = self._makeOne()
         e = invalid_exc(typ.deserialize, None, 'cant.be.found')
-        self.assertEqual(e.msg,
+        self.assertEqual(e.msg.interpolate(),
                          'The dotted name "cant.be.found" cannot be imported')
 
     def test_serialize_ok(self):
@@ -998,13 +1011,21 @@ class TestGlobalObject(unittest.TestCase):
     def test_serialize_fail(self):
         typ = self._makeOne()
         e = invalid_exc(typ.serialize, None, None)
-        self.assertEqual(e.msg, 'None has no __name__')
+        self.assertEqual(e.msg.interpolate(), '"None" has no __name__')
 
 class TestDateTime(unittest.TestCase):
     def _makeOne(self, *arg, **kw):
         from colander import DateTime
         return DateTime(*arg, **kw)
 
+    def _dt(self):
+        import datetime
+        return datetime.datetime(2010, 4, 26, 10, 48)
+
+    def _today(self):
+        import datetime
+        return datetime.date.today()
+
     def test_ctor_default_tzinfo_None(self):
         import iso8601
         typ = self._makeOne()
@@ -1020,12 +1041,13 @@ class TestDateTime(unittest.TestCase):
         typ = self._makeOne()
         node = DummySchemaNode(None)
         e = invalid_exc(typ.serialize, node, 'garbage')
-        self.assertEqual(e.msg, "'garbage' is not a datetime object")
+        self.assertEqual(e.msg.interpolate(),
+                         '"garbage" is not a datetime object')
 
     def test_serialize_with_date(self):
         import datetime
         typ = self._makeOne()
-        date = datetime.date.today()
+        date = self._today()
         node = DummySchemaNode(None)
         result = typ.serialize(node, date)
         expected = datetime.datetime.combine(date, datetime.time())
@@ -1033,19 +1055,17 @@ class TestDateTime(unittest.TestCase):
         self.assertEqual(result, expected)
 
     def test_serialize_with_naive_datetime(self):
-        import datetime
         typ = self._makeOne()
-        dt = datetime.datetime.now()
         node = DummySchemaNode(None)
+        dt = self._dt()
         result = typ.serialize(node, dt)
         expected = dt.replace(tzinfo=typ.default_tzinfo).isoformat()
         self.assertEqual(result, expected)
 
     def test_serialize_with_tzware_datetime(self):
-        import datetime
         import iso8601
         typ = self._makeOne()
-        dt = datetime.datetime.now()
+        dt = self._dt()
         tzinfo = iso8601.iso8601.FixedOffset(1, 0, 'myname')
         dt = dt.replace(tzinfo=tzinfo)
         node = DummySchemaNode(None)
@@ -1056,7 +1076,7 @@ class TestDateTime(unittest.TestCase):
     def test_deserialize_date(self):
         import datetime
         import iso8601
-        date = datetime.date.today()
+        date = self._today()
         typ = self._makeOne()
         formatted = date.isoformat()
         node = DummySchemaNode(None)
@@ -1073,10 +1093,9 @@ class TestDateTime(unittest.TestCase):
         self.failUnless('cannot be parsed' in e.msg)
 
     def test_deserialize_success(self):
-        import datetime
         import iso8601
         typ = self._makeOne()
-        dt = datetime.datetime.now()
+        dt = self._dt()
         tzinfo = iso8601.iso8601.FixedOffset(1, 0, 'myname')
         dt = dt.replace(tzinfo=tzinfo)
         iso = dt.isoformat()
@@ -1089,25 +1108,31 @@ class TestDate(unittest.TestCase):
         from colander import Date
         return Date(*arg, **kw)
 
+    def _dt(self):
+        import datetime
+        return datetime.datetime(2010, 4, 26, 10, 48)
+
+    def _today(self):
+        import datetime
+        return datetime.date.today()
+
     def test_serialize_with_garbage(self):
         typ = self._makeOne()
         node = DummySchemaNode(None)
         e = invalid_exc(typ.serialize, node, 'garbage')
-        self.assertEqual(e.msg, "'garbage' is not a date object")
+        self.assertEqual(e.msg.interpolate(), '"garbage" is not a date object')
 
     def test_serialize_with_date(self):
-        import datetime
         typ = self._makeOne()
-        date = datetime.date.today()
+        date = self._today()
         node = DummySchemaNode(None)
         result = typ.serialize(node, date)
         expected = date.isoformat()
         self.assertEqual(result, expected)
 
     def test_serialize_with_datetime(self):
-        import datetime
         typ = self._makeOne()
-        dt = datetime.datetime.now()
+        dt = self._dt()
         node = DummySchemaNode(None)
         result = typ.serialize(node, dt)
         expected = dt.date().isoformat()
@@ -1140,17 +1165,15 @@ class TestDate(unittest.TestCase):
         self.assertEqual(result, 'abc')
 
     def test_deserialize_success_date(self):
-        import datetime
         typ = self._makeOne()
-        date = datetime.date.today()
+        date = self._today()
         iso = date.isoformat()
         node = DummySchemaNode(None)
         result = typ.deserialize(node, iso)
         self.assertEqual(result.isoformat(), iso)
 
     def test_deserialize_success_datetime(self):
-        import datetime
-        dt = datetime.datetime.now()
+        dt = self._dt()
         typ = self._makeOne()
         iso = dt.isoformat()
         node = DummySchemaNode(None)
